"""
Инициализация utils
"""

from .validators import (
    validate_id, validate_limit, validate_offset, validate_query,
    validate_feed, validate_token, validate_email, validate_password
)
from .helpers import (
    build_params, calculate_retry_delay, sleep_with_jitter,
    format_error_message, extract_api_error
)

__all__ = [
    # Validators
    "validate_id", "validate_limit", "validate_offset", "validate_query",
    "validate_feed", "validate_token", "validate_email", "validate_password",
    
    # Helpers
    "build_params", "calculate_retry_delay", "sleep_with_jitter",
    "format_error_message", "extract_api_error",
]



"""
Исключения для валидации
"""

from .base import ValidationError


class ParameterError(ValidationError):
    """Ошибка параметра"""
    pass


class IDError(ParameterError):
    """Ошибка ID"""
    pass


class LimitError(ParameterError):
    """Ошибка лимита"""
    pass


class QueryError(ParameterError):
    """Ошибка поискового запроса"""
    pass



"""
Модуль для работы с эпизодами
"""

from .base import BaseClient
from ..exceptions.validation import IDError
from ..utils import validate_id
from ..models import Episode, EpisodeResponse


class EpisodesMixin(BaseClient):
    """Миксин для работы с эпизодами"""
    
    async def get_episode(self, episode_id: int) -> Episode:
        """
        Получить информацию об эпизоде
        
        Args:
            episode_id: ID эпизода
            
        Returns:
            Информация об эпизоде
        """
        try:
            episode_id = validate_id(episode_id, "episode_id")
        except ValueError as e:
            raise IDError(str(e))
            
        response = await self._make_request("GET", f"/episodes/{episode_id}")
        return EpisodeResponse(**response).data



"""
Модуль для работы с аниме
"""

from typing import List, Optional
from .base import BaseClient
from ..exceptions.validation import IDError, QueryError
from ..utils import validate_id, validate_limit, validate_offset, validate_query, build_params
from ..models import Series, SeriesResponse


class AnimeMixin(BaseClient):
    """Миксин для работы с аниме"""
    
    async def get_series(
        self,
        limit: int = 20,
        offset: int = 0,
        fields: Optional[List[str]] = None,
        query: Optional[str] = None,
        myAnimeListId: Optional[int] = None,
        chips: Optional[str] = None,
        pretty: bool = False
    ) -> List[Series]:
        """
        Получить список аниме
        
        Args:
            limit: Количество элементов на странице
            offset: Смещение от начала
            fields: Список полей для получения
            query: Поиск по названию
            myAnimeListId: Фильтр по ID на MyAnimeList
            chips: Расширенный фильтр
            pretty: Читабельный формат JSON
            
        Returns:
            Список аниме
        """
        # Валидация параметров
        try:
            limit = validate_limit(limit)
            offset = validate_offset(offset)
            if query:
                query = validate_query(query)
            if myAnimeListId:
                myAnimeListId = validate_id(myAnimeListId, "myAnimeListId")
        except ValueError as e:
            raise QueryError(str(e))
        
        params = build_params(
            limit=limit,
            offset=offset,
            fields=fields,
            query=query,
            myAnimeListId=myAnimeListId,
            chips=chips,
            pretty="1" if pretty else None
        )
        
        response = await self._make_request("GET", "/series/", params)
        data = SeriesResponse(**response).data
        return data if isinstance(data, list) else [data]
        
    async def get_series_by_id(self, series_id: int) -> Series:
        """
        Получить информацию о конкретном аниме
        
        Args:
            series_id: ID аниме
            
        Returns:
            Информация об аниме
        """
        try:
            series_id = validate_id(series_id, "series_id")
        except ValueError as e:
            raise IDError(str(e))
            
        response = await self._make_request("GET", f"/series/{series_id}")
        data = SeriesResponse(**response).data
        return data if isinstance(data, Series) else data[0]
    
    async def search_anime(self, query: str, limit: int = 10) -> List[Series]:
        """
        Поиск аниме по названию
        
        Args:
            query: Поисковый запрос
            limit: Количество результатов
            
        Returns:
            Список найденных аниме
        """
        try:
            query = validate_query(query)
            limit = validate_limit(limit)
        except ValueError as e:
            raise QueryError(str(e))
            
        return await self.get_series(query=query, limit=limit)
    
    async def get_anime_by_mal_id(self, mal_id: int) -> List[Series]:
        """
        Получить аниме по ID MyAnimeList
        
        Args:
            mal_id: ID на MyAnimeList
            
        Returns:
            Список аниме (обычно один элемент)
        """
        try:
            mal_id = validate_id(mal_id, "mal_id")
        except ValueError as e:
            raise IDError(str(e))
            
        return await self.get_series(myAnimeListId=mal_id, limit=1)



"""
Инициализация моделей
"""

from .common import Title, Link, Description, Genre, User
from .anime import Series, SeriesResponse
from .episodes import Episode, EpisodeResponse
from .translations import (
    Translation, TranslationResponse, TranslationsResponse,
    DownloadOption, StreamOption, EmbedData, EmbedResponse,
    UserResponse, LoginResponse
)

# Пересборка моделей для решения циклических ссылок
SeriesResponse.model_rebuild()
EpisodeResponse.model_rebuild()
TranslationResponse.model_rebuild()
TranslationsResponse.model_rebuild()
EmbedResponse.model_rebuild()
UserResponse.model_rebuild()

__all__ = [
    # Common models
    "Title", "Link", "Description", "Genre", "User",
    
    # Anime models
    "Series", "SeriesResponse",
    
    # Episode models
    "Episode", "EpisodeResponse",
    
    # Translation models
    "Translation", "TranslationResponse", "TranslationsResponse",
    "DownloadOption", "StreamOption", "EmbedData", "EmbedResponse",
    "UserResponse", "LoginResponse",
]



"""
Модуль авторизации
"""

from typing import Optional
from .base import BaseClient
from ..exceptions.auth import AuthenticationError, LoginError
from ..utils import validate_token, validate_email, validate_password
from ..models import UserResponse, User


class AuthMixin(BaseClient):
    """Миксин для авторизации"""
    
    def __init__(self, *args, email: Optional[str] = None, password: Optional[str] = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.email = email
        self.password = password
    
    async def login(self) -> str:
        """
        Авторизация по email и паролю
        
        Returns:
            Токен доступа
            
        Raises:
            AuthenticationError: Ошибка авторизации
        """
        if not self.email or not self.password:
            raise AuthenticationError("Email и пароль не указаны")
        
        # Валидация email
        try:
            validate_email(self.email)
        except ValueError as e:
            raise AuthenticationError(str(e))
        
        # Валидация пароля
        try:
            validate_password(self.password)
        except ValueError as e:
            raise AuthenticationError(str(e))
            
        params = {
            "app": "universal",
            "email": self.email,
            "password": self.password
        }
        
        try:
            response = await self._make_request("POST", "/login", params)
            
            if "access_token" not in response:
                raise AuthenticationError("Токен не получен")
            
            token = response["access_token"]
            try:
                validate_token(token)
            except ValueError as e:
                raise AuthenticationError(str(e))
                
            self.access_token = token
            return self.access_token
            
        except Exception as e:
            if isinstance(e, AuthenticationError):
                raise
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise LoginError("Неверный email или пароль")
            else:
                raise AuthenticationError(f"Ошибка авторизации: {e}")
    
    async def get_user_info(self) -> 'User':
        """
        Получить информацию о текущем пользователе
        
        Returns:
            Информация о пользователе
            
        Raises:
            AuthenticationError: Требуется авторизация
        """
        response = await self._make_request("GET", "/me", require_auth=True)
        return UserResponse(**response).data
    
    async def get_access_token_from_site(self) -> str:
        """
        Получить токен доступа через сайт (требует авторизации через браузер)
        
        Примечание: Этот метод требует предварительной авторизации через браузер
        на сайте https://smotret-anime.online/users/login
        
        Returns:
            Токен доступа
            
        Raises:
            AuthenticationError: Требуется авторизация через браузер
        """
        try:
            response = await self._make_request("GET", "/accessToken?app=universal")
            
            if "access_token" not in response:
                raise AuthenticationError("Токен не получен")
            
            token = response["access_token"]
            try:
                validate_token(token)
            except ValueError as e:
                raise AuthenticationError(str(e))
                
            self.access_token = token
            return self.access_token
            
        except Exception as e:
            if isinstance(e, AuthenticationError):
                raise
            else:
                raise AuthenticationError(f"Ошибка получения токена через сайт: {e}")



"""
Anime365 API Wrapper

Асинхронная библиотека для работы с API anime365 (smotret-anime)
"""

__version__ = "1.0.0"
__author__ = "ragedrugg"

# Основной клиент
from .client import Anime365Client

# Исключения
from .exceptions import (
    Anime365Error,
    AuthenticationError, 
    APIError,
    NotFoundError,
    RateLimitError,
    ServerError,
    NetworkError,
    ValidationError,
    TokenError,
    LoginError,
    ParameterError,
    IDError,
    LimitError,
    QueryError
)

# Константы
from .constants import (
    ANIME_TYPES, EPISODE_TYPES, TRANSLATION_TYPES,
    QUALITY_TYPES, LANGUAGE_TYPES, CONTENT_TYPES,
    FEED_TYPES, ACTIVE_STATUS, PREMIUM_STATUS
)

__all__ = [
    # Основной клиент
    "Anime365Client",
    
    # Исключения
    "Anime365Error",
    "AuthenticationError",
    "APIError",
    "NotFoundError",
    "RateLimitError",
    "ServerError",
    "NetworkError",
    "ValidationError",
    "TokenError",
    "LoginError",
    "ParameterError",
    "IDError",
    "LimitError",
    "QueryError",
    
    # Модели данных
    "Series",
    "Episode", 
    "Translation",
    "User",
    "EmbedData",
    "Title",
    "Link",
    "Description",
    "Genre",
    "DownloadOption",
    "StreamOption",
    
    # Константы
    "ANIME_TYPES",
    "EPISODE_TYPES", 
    "TRANSLATION_TYPES",
    "QUALITY_TYPES",
    "LANGUAGE_TYPES",
    "CONTENT_TYPES",
    "FEED_TYPES",
    "ACTIVE_STATUS",
    "PREMIUM_STATUS"
]


"""
Модуль для работы с переводами
"""

from typing import List, Optional
from .base import BaseClient
from ..exceptions.validation import IDError, LimitError, ParameterError
from ..utils import validate_id, validate_limit, validate_offset, validate_feed, build_params
from ..models import (
    Translation, TranslationResponse, TranslationsResponse
)


class TranslationsMixin(BaseClient):
    """Миксин для работы с переводами"""
    
    async def get_translations(
        self,
        feed: str = "recent",
        limit: int = 20,
        offset: int = 0,
        afterId: Optional[int] = None,
        series_id: Optional[int] = None,
        fields: Optional[List[str]] = None
    ) -> List[Translation]:
        """
        Получить список переводов
        
        Args:
            feed: Тип ленты (recent, id, all)
            limit: Количество элементов
            offset: Смещение
            afterId: Получить переводы после указанного ID
            series_id: Фильтр по ID аниме
            fields: Список полей для получения
            
        Returns:
            Список переводов
        """
        # Валидация параметров
        try:
            feed = validate_feed(feed)
            limit = validate_limit(limit)
            offset = validate_offset(offset)
            if afterId:
                afterId = validate_id(afterId, "afterId")
            if series_id:
                series_id = validate_id(series_id, "series_id")
        except ValueError as e:
            raise ParameterError(str(e))
        
        params = build_params(
            feed=feed,
            limit=limit,
            offset=offset,
            afterId=afterId,
            series_id=series_id,
            fields=fields
        )
        
        response = await self._make_request("GET", "/translations/", params)
        return TranslationsResponse(**response).data
        
    async def get_translation(self, translation_id: int) -> Translation:
        """
        Получить информацию о конкретном переводе
        
        Args:
            translation_id: ID перевода
            
        Returns:
            Информация о переводе
        """
        try:
            translation_id = validate_id(translation_id, "translation_id")
        except ValueError as e:
            raise IDError(str(e))
            
        response = await self._make_request("GET", f"/translations/{translation_id}")
        return TranslationResponse(**response).data
    
    async def get_recent_translations(self, limit: int = 20) -> List[Translation]:
        """
        Получить последние переводы онгоингов
        
        Args:
            limit: Количество переводов
            
        Returns:
            Список последних переводов
        """
        try:
            limit = validate_limit(limit)
        except ValueError as e:
            raise LimitError(str(e))
            
        return await self.get_translations(feed="recent", limit=limit)
    
    async def get_all_translations_after(self, after_id: int, limit: int = 1000) -> List[Translation]:
        """
        Получить все переводы после указанного ID (для полного сканирования)
        
        Args:
            after_id: ID после которого получать переводы
            limit: Количество переводов за запрос
            
        Returns:
            Список переводов
        """
        try:
            after_id = validate_id(after_id, "after_id")
            limit = validate_limit(limit)
        except ValueError as e:
            raise ParameterError(str(e))
            
        return await self.get_translations(feed="id", afterId=after_id, limit=limit)
    
    async def get_anime_translations(self, series_id: int, limit: int = 1000) -> List[Translation]:
        """
        Получить все переводы конкретного аниме
        
        Args:
            series_id: ID аниме
            limit: Количество переводов
            
        Returns:
            Список переводов аниме
        """
        try:
            series_id = validate_id(series_id, "series_id")
            limit = validate_limit(limit)
        except ValueError as e:
            raise ParameterError(str(e))
            
        return await self.get_translations(series_id=series_id, limit=limit)



"""
Вспомогательные функции для библиотеки anime365
"""

import asyncio
import random
from typing import Dict, Any, Optional


def build_params(**kwargs) -> Dict[str, Any]:
    """Построение параметров запроса"""
    params = {}
    for key, value in kwargs.items():
        if value is not None:
            if isinstance(value, list):
                params[key] = ",".join(str(v) for v in value)
            else:
                params[key] = value
    return params


def calculate_retry_delay(attempt: int, base_delay: float = 1.0) -> float:
    """Расчет задержки для retry с экспоненциальным backoff и jitter"""
    delay = base_delay * (2 ** attempt)
    jitter = random.uniform(0, 1)
    return delay + jitter


async def sleep_with_jitter(delay: float) -> None:
    """Задержка с небольшим случайным отклонением"""
    jitter = random.uniform(0, 0.1)
    await asyncio.sleep(delay + jitter)


def format_error_message(status_code: int, response_text: str) -> str:
    """Форматирование сообщения об ошибке"""
    if status_code == 401:
        return "Неверный токен авторизации"
    elif status_code == 403:
        return "Доступ запрещен"
    elif status_code == 404:
        return "Ресурс не найден"
    elif status_code == 414:
        return "Запрос слишком длинный"
    elif status_code == 429:
        return "Превышен лимит запросов"
    elif status_code >= 500:
        return f"Ошибка сервера (HTTP {status_code})"
    elif status_code >= 400:
        return f"Ошибка клиента (HTTP {status_code})"
    else:
        return f"Неизвестная ошибка (HTTP {status_code})"


def extract_api_error(response_data: Dict[str, Any]) -> Optional[str]:
    """Извлечение ошибки из ответа API"""
    if isinstance(response_data, dict) and "error" in response_data:
        error_info = response_data["error"]
        if isinstance(error_info, dict) and "code" in error_info:
            code = error_info["code"]
            message = error_info.get("message", "Неизвестная ошибка")
            if code == 404:
                return "Ресурс не найден"
            else:
                return f"API ошибка {code}: {message}"
    return None



"""
Модели для эпизодов
"""

from typing import List, Optional, TYPE_CHECKING
from pydantic import BaseModel, Field
from datetime import datetime

if TYPE_CHECKING:
    from .anime import Series
    from .translations import Translation


class Episode(BaseModel):
    """Модель эпизода аниме"""
    id: int
    episodeFull: Optional[str] = None  # Полное название эпизода
    episodeInt: Optional[str] = None   # Номер эпизода (строка)
    episodeTitle: Optional[str] = None  # Название эпизода
    episodeType: Optional[str] = None   # Тип эпизода
    firstUploadedDateTime: Optional[datetime] = None  # Дата первой загрузки
    isActive: Optional[int] = None      # Активен ли эпизод
    isFirstUploaded: Optional[int] = None  # Первая загрузка
    seriesId: Optional[int] = None      # ID аниме
    translations: Optional[List['Translation']] = Field(default_factory=list)  # Список переводов
    series: Optional['Series'] = None   # Информация об аниме


class EpisodeResponse(BaseModel):
    """Ответ с информацией об эпизоде"""
    data: Episode



"""
Модели для аниме (Series)
"""

from typing import List, Optional, Union, TYPE_CHECKING
from pydantic import BaseModel, Field
from .common import Title, Link, Description, Genre

if TYPE_CHECKING:
    from .episodes import Episode


class Series(BaseModel):
    """Модель аниме (сериала)"""
    id: int
    aniDbId: Optional[int] = None
    animeNewsNetworkId: Optional[int] = None
    fansubsId: Optional[int] = None
    imdbId: Optional[int] = None
    worldArtId: Optional[int] = None
    isActive: Optional[int] = None
    isAiring: Optional[int] = None
    isHentai: Optional[int] = None
    links: List[Link] = Field(default_factory=list)
    myAnimeListId: Optional[int] = None
    myAnimeListScore: Optional[str] = None
    worldArtScore: Optional[str] = None
    worldArtTopPlace: Optional[Union[str, int]] = None
    numberOfEpisodes: Optional[int] = None
    season: Optional[str] = None
    year: Optional[int] = None
    type: Optional[str] = None
    typeTitle: Optional[str] = None
    titles: Optional[Title] = None
    posterUrl: Optional[str] = None
    posterUrlSmall: Optional[str] = None
    titleLines: List[str] = Field(default_factory=list)
    allTitles: List[str] = Field(default_factory=list)
    title: Optional[str] = None
    url: Optional[str] = None
    descriptions: List[Description] = Field(default_factory=list)
    episodes: List['Episode'] = Field(default_factory=list)
    genres: List[Genre] = Field(default_factory=list)


class SeriesResponse(BaseModel):
    """Ответ со списком аниме"""
    data: Union[List[Series], Series]



"""
Исключения для авторизации
"""

from .base import Anime365Error


class AuthenticationError(Anime365Error):
    """Ошибка авторизации"""
    pass


class TokenError(AuthenticationError):
    """Ошибка с токеном"""
    pass


class LoginError(AuthenticationError):
    """Ошибка входа в систему"""
    pass



"""
Модели для переводов
"""

from typing import List, Optional, TYPE_CHECKING
from pydantic import BaseModel, Field
from datetime import datetime

if TYPE_CHECKING:
    from .episodes import Episode
    from .anime import Series
    from .common import User


class Translation(BaseModel):
    """Модель перевода аниме"""
    id: int
    addedDateTime: Optional[datetime] = None      # Дата добавления
    activeDateTime: Optional[datetime] = None     # Дата активации
    authorsList: Optional[List[str]] = Field(default_factory=list)  # Список авторов
    fansubsTranslationId: Optional[int] = None     # ID в базе фансабов
    isActive: Optional[int] = None                 # Активен ли перевод
    priority: Optional[int] = None                 # Приоритет
    qualityType: Optional[str] = None              # Тип качества
    type: Optional[str] = None                    # Тип перевода
    typeKind: Optional[str] = None                 # Вид перевода
    typeLang: Optional[str] = None                 # Язык перевода
    updatedDateTime: Optional[datetime] = None      # Дата обновления
    title: Optional[str] = None                    # Название перевода
    seriesId: Optional[int] = None                 # ID аниме
    episodeId: Optional[int] = None               # ID эпизода
    url: Optional[str] = None                     # URL страницы перевода
    embedUrl: Optional[str] = None                 # URL для embed
    authorsSummary: Optional[str] = None           # Краткое описание авторов
    duration: Optional[float] = None                 # Длительность в секундах
    width: Optional[int] = None                    # Ширина видео
    height: Optional[int] = None                   # Высота видео
    episode: Optional['Episode'] = None            # Информация об эпизоде
    series: Optional['Series'] = None              # Информация об аниме


class TranslationResponse(BaseModel):
    """Ответ с информацией о переводе"""
    data: Translation


class TranslationsResponse(BaseModel):
    """Ответ со списком переводов"""
    data: List[Translation]


class DownloadOption(BaseModel):
    """Опция для скачивания"""
    height: int
    url: Optional[str] = None


class StreamOption(BaseModel):
    """Опция для стриминга"""
    height: int
    urls: List[str] = Field(default_factory=list)


class EmbedData(BaseModel):
    """Данные для embed плеера"""
    embedUrl: Optional[str] = None
    download: List[DownloadOption] = Field(default_factory=list)
    stream: List[StreamOption] = Field(default_factory=list)
    subtitlesUrl: Optional[str] = None
    subtitlesVttUrl: Optional[str] = None


class EmbedResponse(BaseModel):
    """Ответ с embed данными"""
    data: EmbedData


class UserResponse(BaseModel):
    """Ответ с информацией о пользователе"""
    data: 'User'


class LoginResponse(BaseModel):
    """Ответ авторизации"""
    access_token: str



"""
Исключения для библиотеки
"""

from .base import Anime365Error, ValidationError, NetworkError
from .auth import AuthenticationError, TokenError, LoginError
from .api import APIError, NotFoundError, RateLimitError, ServerError, ClientError
from .validation import ParameterError, IDError, LimitError, QueryError

__all__ = [
    # Base exceptions
    "Anime365Error",
    "ValidationError", 
    "NetworkError",
    
    # Auth exceptions
    "AuthenticationError",
    "TokenError",
    "LoginError",
    
    # API exceptions
    "APIError",
    "NotFoundError",
    "RateLimitError",
    "ServerError",
    "ClientError",
    
    # Validation exceptions
    "ParameterError",
    "IDError",
    "LimitError",
    "QueryError",
]



"""
Базовые исключения для библиотеки
"""


class Anime365Error(Exception):
    """Базовое исключение для всех ошибок библиотеки"""
    pass


class ValidationError(Anime365Error):
    """Ошибка валидации входных данных"""
    pass


class NetworkError(Anime365Error):
    """Ошибка сети"""
    pass



"""
Валидаторы для библиотеки anime365
"""

from typing import Any


def validate_id(value: Any, name: str = "ID") -> int:
    """Валидация ID"""
    if not isinstance(value, int) or value < 1:
        raise ValueError(f"{name} должен быть положительным числом")
    return value


def validate_limit(value: Any) -> int:
    """Валидация лимита"""
    if not isinstance(value, int):
        raise ValueError("limit должен быть числом")
    if value < 0 or value > 1000:
        raise ValueError("limit должен быть между 0 и 1000")
    return value


def validate_offset(value: Any) -> int:
    """Валидация смещения"""
    if not isinstance(value, int):
        raise ValueError("offset должен быть числом")
    if value < 0:
        raise ValueError("offset не может быть отрицательным")
    return value


def validate_query(value: Any) -> str:
    """Валидация поискового запроса"""
    if not isinstance(value, str):
        raise ValueError("query должен быть строкой")
    if not value or not value.strip():
        raise ValueError("query не может быть пустым")
    if len(value) > 1000:
        raise ValueError("query слишком длинный")
    return value.strip()


def validate_feed(value: Any) -> str:
    """Валидация типа ленты"""
    if not isinstance(value, str):
        raise ValueError("feed должен быть строкой")
    if value not in ["recent", "id", "all"]:
        raise ValueError("feed должен быть 'recent', 'id' или 'all'")
    return value


def validate_token(value: Any) -> str:
    """Валидация токена"""
    if not isinstance(value, str):
        raise ValueError("access_token должен быть строкой")
    if len(value) < 10:
        raise ValueError("access_token должен быть строкой длиной не менее 10 символов")
    return value


def validate_email(value: Any) -> str:
    """Валидация email"""
    if not isinstance(value, str):
        raise ValueError("email должен быть строкой")
    if "@" not in value or len(value) < 5:
        raise ValueError("Неверный формат email")
    return value


def validate_password(value: Any) -> str:
    """Валидация пароля"""
    if not isinstance(value, str):
        raise ValueError("password должен быть строкой")
    if len(value) < 3:
        raise ValueError("Пароль слишком короткий")
    return value



"""
Константы для типов данных API anime365
"""

# Типы аниме (type)
ANIME_TYPES = {
    "tv": "ТВ сериал",
    "movie": "Фильм", 
    "ova": "OVA",
    "ona": "ONA",
    "special": "Спешл",
    "music": "Музыкальное видео"
}

# Типы эпизодов (episodeType)
EPISODE_TYPES = {
    "tv": "ТВ эпизод",
    "preview": "Превью",
    "special": "Спешл",
    "ova": "OVA",
    "ona": "ONA"
}

# Типы переводов (type)
TRANSLATION_TYPES = {
    "subRu": "Русские субтитры",
    "subEn": "Английские субтитры", 
    "voiceRu": "Русская озвучка",
    "voiceEn": "Английская озвучка",
    "raw": "RAW (без перевода)"
}

# Типы качества (qualityType)
QUALITY_TYPES = {
    "tv": "ТВ качество",
    "bd": "Blu-ray качество",
    "dvd": "DVD качество"
}

# Типы языков (typeLang)
LANGUAGE_TYPES = {
    "ru": "Русский",
    "en": "Английский",
    "ja": "Японский"
}

# Типы контента (typeKind)
CONTENT_TYPES = {
    "sub": "Субтитры",
    "voice": "Озвучка", 
    "raw": "RAW"
}

# Типы лент для переводов (feed)
FEED_TYPES = {
    "recent": "Последние переводы онгоингов",
    "id": "Все переводы по ID (в начале самые старые)",
    "all": "Все переводы включая неактивные"
}

# Статусы активности
ACTIVE_STATUS = {
    0: "Неактивно",
    1: "Активно"
}

# Статусы премиума
PREMIUM_STATUS = {
    False: "Обычный пользователь",
    True: "Премиум пользователь"
}



"""
Модуль для работы с embed данными
"""

from .base import BaseClient
from ..exceptions.auth import AuthenticationError
from ..exceptions.validation import IDError
from ..utils import validate_id
from ..models import EmbedData, EmbedResponse


class EmbedMixin(BaseClient):
    """Миксин для работы с embed данными"""
    
    async def get_embed_data(self, translation_id: int) -> EmbedData:
        """
        Получить embed информацию для воспроизведения/скачивания
        
        Args:
            translation_id: ID перевода
            
        Returns:
            Embed данные
            
        Raises:
            AuthenticationError: Требуется авторизация
        """
        try:
            translation_id = validate_id(translation_id, "translation_id")
        except ValueError as e:
            raise IDError(str(e))
        
        if not self.access_token:
            raise AuthenticationError("Требуется авторизация для получения embed данных")
            
        response = await self._make_request(
            "GET", 
            f"/translations/embed/{translation_id}",
            require_auth=True
        )
        return EmbedResponse(**response).data



"""
Основной клиент anime365
"""

from typing import Optional
from aiohttp import ClientSession

from .base import BaseClient
from .auth import AuthMixin
from .anime import AnimeMixin
from .episodes import EpisodesMixin
from .translations import TranslationsMixin
from .embed import EmbedMixin


class Anime365Client(
    AuthMixin,
    AnimeMixin,
    EpisodesMixin,
    TranslationsMixin,
    EmbedMixin,
    BaseClient
):
    """
    Асинхронный клиент для работы с API anime365 (smotret-anime.online)
    
    Объединяет все возможности библиотеки в одном классе.
    """
    
    def __init__(
        self,
        access_token: Optional[str] = None,
        email: Optional[str] = None,
        password: Optional[str] = None,
        timeout: int = 30,
        session: Optional[ClientSession] = None,
        max_retries: int = 3,
        retry_delay: float = 1.0
    ):
        """
        Инициализация клиента
        
        Args:
            access_token: Токен доступа (если есть)
            email: Email для авторизации
            password: Пароль для авторизации
            timeout: Таймаут запросов в секундах
            session: Существующая сессия aiohttp (опционально)
            max_retries: Максимальное количество повторных попыток
            retry_delay: Задержка между повторными попытками в секундах
        """
        # Валидация токена при инициализации
        if access_token is not None:
            from ..utils import validate_token
            try:
                validate_token(access_token)
            except ValueError as e:
                from ..exceptions import TokenError
                raise TokenError(str(e))
        
        super().__init__(
            access_token=access_token,
            timeout=timeout,
            session=session,
            max_retries=max_retries,
            retry_delay=retry_delay,
            email=email,
            password=password
        )



"""
Общие модели данных
"""

from typing import Optional
from pydantic import BaseModel
from datetime import datetime


class Title(BaseModel):
    """Названия аниме на разных языках"""
    ru: Optional[str] = None
    romaji: Optional[str] = None
    ja: Optional[str] = None
    en: Optional[str] = None


class Link(BaseModel):
    """Ссылка на внешний ресурс"""
    title: str
    url: Optional[str] = None  # Может быть относительным URL
    
    @classmethod
    def validate_url(cls, v):
        """Валидация URL - принимает как абсолютные, так и относительные URL"""
        if v is None:
            return None
        if isinstance(v, str):
            # Если это относительный URL, оставляем как есть
            if not v.startswith(('http://', 'https://')):
                return v
            # Если это абсолютный URL, валидируем
            try:
                from pydantic import HttpUrl
                return str(HttpUrl(v))
            except:
                return v
        return v


class Description(BaseModel):
    """Описание аниме из источника"""
    source: str
    value: str
    updatedDateTime: datetime


class Genre(BaseModel):
    """Жанр аниме"""
    id: int
    title: str
    url: Optional[str] = None  # Может быть относительным URL


class User(BaseModel):
    """Модель пользователя"""
    isLogined: bool
    id: Optional[int] = None              # ID пользователя
    name: Optional[str] = None            # Имя пользователя
    isPremium: Optional[bool] = None      # Премиум ли пользователь
    premiumUntil: Optional[str] = None    # До какого числа действует премиум



"""
Исключения для API
"""

from .base import Anime365Error


class APIError(Anime365Error):
    """Ошибка API"""
    pass


class NotFoundError(APIError):
    """Ресурс не найден (404)"""
    pass


class RateLimitError(APIError):
    """Превышен лимит запросов (429)"""
    pass


class ServerError(APIError):
    """Ошибка сервера (5xx)"""
    pass


class ClientError(APIError):
    """Ошибка клиента (4xx)"""
    pass



"""
Базовый клиент для anime365
"""

import logging
from typing import Optional, Dict, Any
from aiohttp import ClientSession, ClientTimeout

from ..exceptions.base import NetworkError
from ..exceptions.auth import AuthenticationError
from ..exceptions.api import APIError, NotFoundError, RateLimitError, ServerError
from ..utils import (
    calculate_retry_delay, sleep_with_jitter, 
    format_error_message, extract_api_error
)


class BaseClient:
    """Базовый класс клиента"""
    
    BASE_URL = "https://smotret-anime.online/api"
    USER_AGENT = "anime365-wrapper-python/1.0.0"
    
    def __init__(
        self,
        access_token: Optional[str] = None,
        timeout: int = 30,
        session: Optional[ClientSession] = None,
        max_retries: int = 3,
        retry_delay: float = 1.0
    ):
        """
        Инициализация базового клиента
        
        Args:
            access_token: Токен доступа
            timeout: Таймаут запросов в секундах
            session: Существующая сессия aiohttp
            max_retries: Максимальное количество повторных попыток
            retry_delay: Задержка между повторными попытками в секундах
        """
        self.access_token = access_token
        self.timeout = ClientTimeout(total=timeout)
        self._session = session
        self._own_session = session is None
        self.max_retries = max_retries
        self.retry_delay = retry_delay
        
        # Настройка логирования
        self.logger = logging.getLogger(__name__)
    
    async def _ensure_session(self):
        """Обеспечить наличие сессии"""
        if self._session is None:
            self._session = ClientSession(
                timeout=self.timeout,
                headers={"User-Agent": self.USER_AGENT}
            )
    
    async def _make_request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict[str, Any]] = None,
        require_auth: bool = False,
        retry_count: int = 0
    ) -> Dict[str, Any]:
        """
        Выполнить HTTP запрос к API с retry механизмом
        
        Args:
            method: HTTP метод (GET, POST)
            endpoint: Endpoint API
            params: Параметры запроса
            require_auth: Требуется ли авторизация
            retry_count: Текущее количество попыток
            
        Returns:
            JSON ответ от API
            
        Raises:
            Anime365Error: Ошибка API
            AuthenticationError: Ошибка авторизации
        """
        await self._ensure_session()
        
        url = f"{self.BASE_URL}{endpoint}"
        
        # Добавляем токен если требуется авторизация
        if require_auth and self.access_token:
            if params is None:
                params = {}
            params["access_token"] = self.access_token
            
        # Выполняем запрос с retry механизмом
        try:
            async with self._session.request(
                method=method,
                url=url,
                params=params
            ) as response:
                # Обработка различных статус кодов
                if response.status == 401:
                    raise AuthenticationError("Неверный токен авторизации")
                elif response.status == 403:
                    raise AuthenticationError("Доступ запрещен")
                elif response.status == 404:
                    raise NotFoundError("Ресурс не найден")
                elif response.status == 414:
                    raise APIError("Запрос слишком длинный")
                elif response.status == 429:
                    # Rate limit - пробуем повторить
                    if retry_count < self.max_retries:
                        delay = calculate_retry_delay(retry_count, self.retry_delay)
                        self.logger.warning(f"Rate limit hit, retrying in {delay:.2f}s (attempt {retry_count + 1})")
                        await sleep_with_jitter(delay)
                        return await self._make_request(method, endpoint, params, require_auth, retry_count + 1)
                    else:
                        raise RateLimitError("Превышен лимит запросов")
                elif response.status >= 500:
                    # Серверные ошибки - пробуем повторить
                    if retry_count < self.max_retries:
                        delay = calculate_retry_delay(retry_count, self.retry_delay)
                        self.logger.warning(f"Server error {response.status}, retrying in {delay:.2f}s (attempt {retry_count + 1})")
                        await sleep_with_jitter(delay)
                        return await self._make_request(method, endpoint, params, require_auth, retry_count + 1)
                    else:
                        error_text = await response.text()
                        raise ServerError(f"HTTP {response.status}: {error_text}")
                elif response.status >= 400:
                    error_text = await response.text()
                    raise APIError(format_error_message(response.status, error_text))
                    
                response_data = await response.json()
                
                # Проверяем на наличие ошибок в ответе
                api_error = extract_api_error(response_data)
                if api_error:
                    raise APIError(api_error)
                
                return response_data
                
        except Exception as e:
            if isinstance(e, (AuthenticationError, APIError, NotFoundError, RateLimitError, ServerError)):
                raise
            # Сетевые ошибки - пробуем повторить
            if retry_count < self.max_retries:
                delay = calculate_retry_delay(retry_count, self.retry_delay)
                self.logger.warning(f"Network error: {e}, retrying in {delay:.2f}s (attempt {retry_count + 1})")
                await sleep_with_jitter(delay)
                return await self._make_request(method, endpoint, params, require_auth, retry_count + 1)
            else:
                raise NetworkError(f"Ошибка сети: {e}")
    
    async def __aenter__(self):
        """Async context manager entry"""
        if self._own_session:
            await self._ensure_session()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit"""
        if self._own_session and self._session:
            await self._session.close()
    
    async def close(self):
        """Закрыть сессию"""
        if self._own_session and self._session:
            await self._session.close()
            self._session = None
